@@ -23,20 +23,22 @@
 
 #include "qtcompat.h"
 
-#include <QtCore/QDir>
-#include <QtCore/QFile>
-#include <QtCore/QFileInfo>
-#include <QtCore/QRegularExpression>
+#include <QtCore/qdir.h>
+#include <QtCore/qfile.h>
+#include <QtCore/qfileinfo.h>
+#include <QtCore/qregularexpression.h>
 
 using namespace Qt::StringLiterals;
 
-static const char ENABLE_PYSIDE_EXTENSIONS[] = "enable-pyside-extensions";
-static const char AVOID_PROTECTED_HACK[] = "avoid-protected-hack";
+static constexpr auto ENABLE_PYSIDE_EXTENSIONS = "enable-pyside-extensions"_L1;
+static constexpr auto AVOID_PROTECTED_HACK = "avoid-protected-hack"_L1;
+static constexpr auto DISABLED_OPTIMIZATIONS = "unoptimize"_L1;
 
 struct GeneratorOptions
 {
     bool usePySideExtensions = false;
     bool avoidProtectedHack = false;
+    Generator::CodeOptimization optimizations = Generator::AllCodeOptimizations;
 };
 
 struct Generator::GeneratorPrivate
@@ -52,6 +54,9 @@ struct Generator::GeneratorPrivate
 
 GeneratorOptions Generator::GeneratorPrivate::m_options;
 
+// Kept as a variable for a potential Qt-in-namespace support
+QString Generator::m_gsp = "::"_L1;
+
 Generator::Generator() : m_d(new GeneratorPrivate)
 {
 }
@@ -89,11 +94,13 @@ bool Generator::setup(const ApiExtractorResult &api)
 QList<OptionDescription> Generator::options()
 {
     return {
-        {QLatin1StringView(AVOID_PROTECTED_HACK),
+        {AVOID_PROTECTED_HACK,
          u"Avoid the use of the '#define protected public' hack."_s},
-        {QLatin1StringView(ENABLE_PYSIDE_EXTENSIONS),
+        {ENABLE_PYSIDE_EXTENSIONS,
          u"Enable PySide extensions, such as support for signal/slots,\n"
-          "use this if you are creating a binding for a Qt-based library."_s}
+          "use this if you are creating a binding for a Qt-based library."_s},
+        {DISABLED_OPTIMIZATIONS,
+         "Disable optimization options"_L1}
     };
 }
 
@@ -103,6 +110,7 @@ public:
     explicit GeneratorOptionsParser(GeneratorOptions *o) : m_options(o) {}
 
     bool handleBoolOption(const QString &key, OptionSource source) override;
+    bool handleOption(const QString &key, const QString &value, OptionSource source) override;
 
 private:
     GeneratorOptions *m_options;
@@ -112,34 +120,66 @@ bool GeneratorOptionsParser::handleBoolOption(const QString & key, OptionSource
 {
     if (source == OptionSource::CommandLineSingleDash)
         return false;
-    if (key == QLatin1StringView(ENABLE_PYSIDE_EXTENSIONS))
+    if (key == ENABLE_PYSIDE_EXTENSIONS)
         return ( m_options->usePySideExtensions = true);
-    if (key == QLatin1StringView(AVOID_PROTECTED_HACK))
+    if (key == AVOID_PROTECTED_HACK)
         return ( m_options->avoidProtectedHack = true);
     return false;
 }
 
+bool GeneratorOptionsParser::handleOption(const QString & key, const QString & value,
+                                          OptionSource source)
+{
+    if (source == OptionSource::CommandLineSingleDash)
+        return false;
+
+    if (key == DISABLED_OPTIMIZATIONS) {
+        const auto tokens = QStringView{value}.split(u',');
+        for (const auto &tok : tokens) {
+            if (tok == "fullname"_L1)
+                m_options->optimizations.setFlag(Generator::RemoveFullnameField, false);
+            else if (tok == "compression"_L1)
+                m_options->optimizations.setFlag(Generator::CompressSignatureStrings, false);
+            else if (tok == "folding"_L1)
+                m_options->optimizations.setFlag(Generator::FoldCommonTailCode, false);
+            else if (tok == "all"_L1)
+                m_options->optimizations = Generator::CodeOptimization(0);
+            else
+                return false;
+        }
+        return true;
+    }
+
+    return false;
+}
+
 std::shared_ptr<OptionsParser> Generator::createOptionsParser()
 {
     return std::make_shared<GeneratorOptionsParser>(&GeneratorPrivate::m_options);
 }
 
+QString Generator::fileNameForClassHelper(const AbstractMetaClassCPtr &metaClass,
+                                          const QString &suffix,
+                                          FileNameFlags flags)
+{
+    QString fileNameBase = flags.testFlag(FileNameFlag::UnqualifiedName)
+        ? metaClass->name() : metaClass->qualifiedCppName();
+    if (!flags.testFlag(FileNameFlag::KeepCase))
+        fileNameBase = fileNameBase.toLower();
+    fileNameBase.replace(u"::"_s, u"_"_s);
+    return fileNameBase + suffix;
+}
+
 QString Generator::fileNameForContextHelper(const GeneratorContext &context,
                                             const QString &suffix,
                                             FileNameFlags flags)
 
 {
-    if (!context.forSmartPointer()) {
-        const auto metaClass = context.metaClass();
-        QString fileNameBase = flags.testFlag(FileNameFlag::UnqualifiedName)
-            ? metaClass->name() : metaClass->qualifiedCppName();
-        if (!flags.testFlag(FileNameFlag::KeepCase))
-            fileNameBase = fileNameBase.toLower();
-        fileNameBase.replace(u"::"_s, u"_"_s);
-        return fileNameBase + suffix;
-    }
+    if (!context.forSmartPointer())
+        return fileNameForClassHelper(context.metaClass(), suffix, flags);
 
-    // FIXME: PYSIDE7: Use the above code path for all types. Note the file
+    // FIXME: PYSIDE7: Use the above code path for all types and change
+    // parameter type to AbstractMetaClassCPtr. Note the file
     // names will then change to reflect the namespaces of the pointee
     // (smart/integer2).
     const AbstractMetaType &smartPointerType = context.preciseType();
@@ -200,27 +240,16 @@ void Generator::setOutputDirectory(const QString &outDir)
     m_d->outDir = outDir;
 }
 
-bool Generator::generateFileForContext(const GeneratorContext &context)
+QString Generator::directoryForContext(const GeneratorContext &context) const
 {
-    const auto cls = context.metaClass();
-    auto typeEntry = cls->typeEntry();
+    return m_d->outDir + u'/'
+        + subDirectoryForPackage(context.metaClass()->typeEntry()->targetLangPackage());
+}
 
-    if (!shouldGenerate(typeEntry))
-        return true;
-
-    const QString fileName = fileNameForContext(context);
-    if (fileName.isEmpty())
-        return true;
-
-    QString filePath = outputDirectory() + u'/'
-        + subDirectoryForPackage(typeEntry->targetLangPackage())
-        + u'/' + fileName;
-    FileOut fileOut(filePath);
-
-    generateClass(fileOut.stream, context);
-
-    fileOut.done();
-    return true;
+void Generator::generateSmartPointerClass(TextStream &,
+                                          const QString &,
+                                          const GeneratorContext &)
+{
 }
 
 QString Generator::getFileNameBaseForSmartPointer(const AbstractMetaType &smartPointerType)
@@ -228,16 +257,16 @@ QString Generator::getFileNameBaseForSmartPointer(const AbstractMetaType &smartP
     const AbstractMetaType innerType = smartPointerType.getSmartPointerInnerType();
     smartPointerType.typeEntry()->qualifiedCppName();
     QString fileName = smartPointerType.typeEntry()->qualifiedCppName().toLower();
-    fileName.replace(u"::"_s, u"_"_s);
-    fileName.append(u"_"_s);
+    fileName.append(u'_');
     fileName.append(innerType.name().toLower());
-
+    fileName.replace(u"::"_s, u"_"_s); // std::shared_ptr<std::string>
     return fileName;
 }
 
 GeneratorContext Generator::contextForClass(const AbstractMetaClassCPtr &c) const
 {
     GeneratorContext result;
+    result.m_type = GeneratorContext::Class;
     result.m_metaClass = c;
     return result;
 }
@@ -257,22 +286,34 @@ GeneratorContext
 
 bool Generator::generate()
 {
+    QList<GeneratorContext> contexts;
+    contexts.reserve(m_d->api.classes().size());
+
     for (const auto &cls : m_d->api.classes()) {
-        if (!generateFileForContext(contextForClass(cls)))
-            return false;
         auto te = cls->typeEntry();
-        if (shouldGenerate(te) && te->isPrivate())
-            m_d->m_hasPrivateClasses = true;
+        if (shouldGenerate(te)) {
+            contexts.append(contextForClass(cls));
+            if (te->isPrivate())
+                m_d->m_hasPrivateClasses = true;
+        }
     }
 
+    while (!contexts.isEmpty()) {
+        const auto context = contexts.takeFirst();
+        const QString targetDirectory = directoryForContext(context);
+        FileOut fileOut(targetDirectory + u'/' + fileNameForContext(context));
+        generateClass(fileOut.stream, targetDirectory, context, &contexts);
+        fileOut.done();
+    }
+
+    // Generate smart pointers.
     for (const auto &smp: m_d->api.instantiatedSmartPointers()) {
-        AbstractMetaClassCPtr pointeeClass;
-        const auto instantiatedType = smp.type.instantiations().constFirst().typeEntry();
-        if (instantiatedType->isComplex()) // not a C++ primitive
-            pointeeClass = AbstractMetaClass::findClass(m_d->api.classes(), instantiatedType);
-        if (!generateFileForContext(contextForSmartPointer(smp.specialized, smp.type,
-                                                           pointeeClass))) {
-            return false;
+        if (shouldGenerate(smp.specialized->typeEntry())) {
+            const auto context = contextForSmartPointer(smp.specialized, smp.type, smp.pointee);
+            const QString targetDirectory = directoryForContext(context);
+            FileOut fileOut(targetDirectory + u'/' + fileNameForContext(context));
+            generateSmartPointerClass(fileOut.stream, targetDirectory, context);
+            fileOut.done();
         }
     }
     return finishGeneration();
@@ -303,14 +344,17 @@ bool Generator::avoidProtectedHack()
     return GeneratorPrivate::m_options.avoidProtectedHack;
 }
 
+Generator::CodeOptimization Generator::optimizations()
+{
+    return GeneratorPrivate::m_options.optimizations;
+}
+
 QString Generator::getFullTypeName(TypeEntryCPtr type)
 {
     QString result = type->qualifiedCppName();
     if (type->isArray())
         type = std::static_pointer_cast<const ArrayTypeEntry>(type)->nestedTypeEntry();
-    if (!isCppPrimitive(type))
-        result.prepend(u"::"_s);
-    return result;
+    return isCppPrimitive(type) ? result : addGlobalScopePrefix(result);
 }
 
 QString Generator::getFullTypeName(const AbstractMetaType &type)
@@ -320,18 +364,20 @@ QString Generator::getFullTypeName(const AbstractMetaType &type)
     if (type.isVoidPointer())
         return u"void*"_s;
     if (type.typeEntry()->isContainer())
-        return u"::"_s + type.cppSignature();
+        return addGlobalScopePrefix(type.cppSignature());
     QString typeName;
     if (type.typeEntry()->isComplex() && type.hasInstantiations())
         typeName = getFullTypeNameWithoutModifiers(type);
     else
         typeName = getFullTypeName(type.typeEntry());
-    return typeName + QString::fromLatin1("*").repeated(type.indirections());
+    return typeName + QString(type.indirections(), u'*');
 }
 
 QString Generator::getFullTypeName(const AbstractMetaClassCPtr &metaClass)
 {
-    return u"::"_s + metaClass->qualifiedCppName();
+    const QString &qualName = metaClass->qualifiedCppName();
+    // Typedefs are generated into the global namespace
+    return metaClass->isTypeDef() ? qualName : addGlobalScopePrefix(qualName);
 }
 
 QString Generator::getFullTypeNameWithoutModifiers(const AbstractMetaType &type)
@@ -357,7 +403,7 @@ QString Generator::getFullTypeNameWithoutModifiers(const AbstractMetaType &type)
     }
     while (typeName.endsWith(u'*') || typeName.endsWith(u' '))
         typeName.chop(1);
-    return u"::"_s + typeName;
+    return addGlobalScopePrefix(typeName);
 }
 
 std::optional<DefaultValue>
@@ -386,7 +432,7 @@ std::optional<DefaultValue>
     if (type.isNativePointer())
         return DefaultValue(DefaultValue::Pointer, type.typeEntry()->qualifiedCppName());
     if (type.isPointer())
-        return DefaultValue(DefaultValue::Pointer, u"::"_s + type.typeEntry()->qualifiedCppName());
+        return DefaultValue(DefaultValue::Pointer, getFullTypeName(type.typeEntry()));
 
     if (type.typeEntry()->isSmartPointer())
         return minimalConstructor(api, type.typeEntry());
@@ -435,8 +481,7 @@ std::optional<DefaultValue>
         if (const auto nullValue = enumEntry->nullValue())
             return DefaultValue(DefaultValue::Enum, nullValue->name());
         return DefaultValue(DefaultValue::Custom,
-                            u"static_cast< ::"_s + type->qualifiedCppName()
-                            + u">(0)"_s);
+                            "static_cast< "_L1 + getFullTypeName(type) + ">(0)"_L1);
     }
 
     if (type->isFlags()) {
@@ -646,8 +691,18 @@ QString Generator::subDirectoryForPackage(QString packageNameIn) const
     return packageNameIn;
 }
 
+QString Generator::addGlobalScopePrefix(const QString &t)
+{
+    return t.startsWith("std::"_L1) ? t : m_gsp + t;
+}
+
+QString Generator::globalScopePrefix(const GeneratorContext &classContext)
+{
+    return classContext.useWrapper() ? QString{} : m_gsp;
+}
+
 template<typename T>
-static QString getClassTargetFullName_(T t, bool includePackageName)
+static QString getClassTargetFullName_(const T &t, bool includePackageName)
 {
     QString name = t->name();
     AbstractMetaClassCPtr context = t->enclosingClass();
